/*
A simple graphics library for CSE 20211 by Douglas Thain

This work is licensed under a Creative Commons Attribution 4.0 International License.  https://creativecommons.org/licenses/by/4.0/

For complete documentation, see:
http://www.nd.edu/~dthain/courses/cse20211/fall2013/gfx
*/

#include <X11/Xlib.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>

extern "C"
{

	static Display *display=0;
	static Window  window;
	static GC      gc;
	static Colormap colormap;
	static int      fast_color_mode = 0;

	/* These values are saved by wait then retrieved later by xpos and ypos. */

	static int saved_xpos = 0;
	static int saved_ypos = 0;

	/* Flush all previous output to the window. */

	void flush()
	{
		XFlush(display);
	}

	/* Open a new graphics window. */

	void openWindow(char *title, int width = 400, int height = 400)
	{
		display = XOpenDisplay(0);
		if(!display) {
			fprintf(stderr,"open: unable to open the graphics window.\n");
			exit(1);
		}

		Visual *visual = DefaultVisual(display,0);
		if(visual /*&& visual->class==TrueColor*/) {
			fast_color_mode = 1;
		} else {
			fast_color_mode = 0;
		}

		int blackColor = BlackPixel(display, DefaultScreen(display));
		int whiteColor = WhitePixel(display, DefaultScreen(display));

		window = XCreateSimpleWindow(display, DefaultRootWindow(display), 0, 0, width, height, 0, blackColor, blackColor);

		XSetWindowAttributes attr;
		attr.backing_store = Always;

		XChangeWindowAttributes(display,window,CWBackingStore,&attr);

		XStoreName(display,window,title);

		XSelectInput(display, window, StructureNotifyMask|KeyPressMask|ButtonPressMask);

		XMapWindow(display,window);

		gc = XCreateGC(display, window, 0, 0);

		colormap = DefaultColormap(display,0);

		XSetForeground(display, gc, whiteColor);

		// Wait for the MapNotify event

		for(;;) {
			XEvent e;
			XNextEvent(display, &e);
			if (e.type == MapNotify)
				break;
		}
	}

	/* Draw a single point at (x,y) */

	#define drawPixel(x,y) XDrawPoint(display,window,gc,x,y)
    #define setPixel(x,y) XDrawPoint(display,window,gc,x,y)
	/* Draw a line from (x1,y1) to (x2,y2) */

	void drawLine( int x1, int y1, int x2, int y2 )
	{
		XDrawLine(display,window,gc,x1,y1,x2,y2);
	}

	void drawBox(int x0, int y0, int x1, int y1)
	{
		for (int t=x0; t<x1;t++){
			drawLine(t,y0,t,y1);
		}
	}

	void drawCircle(int x, int y, int radius)
	{
		for (double t=0; t<10; t+=0.01){
			setPixel(x+(int)(radius*sin(t)),y+(int)(radius*cos(t)));
		}
	}
	/* Change the current drawing color. */

	void setColor( int r, int g, int b )
	{
		XColor color;

		if(fast_color_mode) {
			/* If this is a truecolor display, we can just pick the color directly. */
			color.pixel = ((b&0xff) | ((g&0xff)<<8) | ((r&0xff)<<16) );
		} else {
			/* Otherwise, we have to allocate it from the colormap of the display. */
			color.pixel = 0;
			color.red = r<<8;
			color.green = g<<8;
			color.blue = b<<8;
			XAllocColor(display,colormap,&color);
		}

		XSetForeground(display, gc, color.pixel);
	}

	/* Clear the graphics window to the background color. */

	void clearScreen()
	{
		XClearWindow(display,window);
	}

	/* Change the current background color. */

	void clearColor( int r, int g, int b )
	{
		XColor color;
		color.pixel = 0;
		color.red = r<<8;
		color.green = g<<8;
		color.blue = b<<8;
		XAllocColor(display,colormap,&color);

		XSetWindowAttributes attr;
		attr.background_pixel = color.pixel;
		XChangeWindowAttributes(display,window,CWBackPixel,&attr);
	}

	int waitEvent()
	{
	       XEvent event;

	       flush();

	       while (1) {
	               if(XCheckMaskEvent(display,-1,&event)) {
	                       if(event.type==KeyPress) {
	                               XPutBackEvent(display,&event);
	                               return 1;
	                       } else if (event.type==ButtonPress) {
	                               XPutBackEvent(display,&event);
	                               return 1;
	                       } else {
	                               return 0;
	                       }
	               } else {
	                       return 0;
	               }
	       }
	}

	/* Wait for the user to press a key or mouse button. */

	char keyPressed()
	{
		XEvent event;

		flush();

		while(1) {
			XNextEvent(display,&event);

			if(event.type==KeyPress) {
				saved_xpos = event.xkey.x;
				saved_ypos = event.xkey.y;
				return XLookupKeysym(&event.xkey,0);
			} else if(event.type==ButtonPress) {
				saved_xpos = event.xkey.x;
				saved_ypos = event.xkey.y;
				return event.xbutton.button;
			}
		}
	}

	/* Return the X and Y coordinates of the last event. */

	int xpos()
	{
		return saved_xpos;
	}

	int ypos()
	{
		return saved_ypos;
	}

}
